function [rho_new] =regression_multivariate
%This function regress surface DH to density
close all


load data.mat
load mean_fields.mat
dep_ax8 = dep_ax8(1,:);


out_file = 'P_corr_DH_RHO_lev_AR1.mat' ;modo = 'SIGMA';
%Calculate regression terms and store in pDH_RHO
[pDH_RHO,rho_new,Y_st]=regress_fields_mvreg(lat_ax8,DH_ax8,d0_m,rho,rho_m,modo);

%Reconstruct fields using stored pDH_RHO
[rho_rec,C_RHO_DH] = reconstruct_fields_mvreg(lat_ax8,DH_ax8,d0_m,rho,rho_m,pDH_RHO,Y_st,modo);

%Save fitted data
save(out_file,'pDH_RHO','C_RHO_DH','rho_new','rho','rho_rec','lat_ax8','Y_st')

%Compare fields
plot_fields(dep_ax8,lat_ax8,rho,rho_new,rho_rec)

%Plot reconstruction in 2 depths
plot_regression_fit(rho_rec,rho,dep_ax8,modo)

%Plot correlation of fitted and original fields
plot_C_RHO_DH(C_RHO_DH,lat_ax8,dep_ax8)


end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [pDH_RHO,rho_new,Y_st]=regress_fields_mvreg(lat,p1_0,p1_m,p2_0,p2_m,modo)
%%Calculate regression coefficients
[nz,nval,nl]=size(p2_0);
nani = 1:nval;
disp(['Calculate regression coefficients - ',upper(modo)])
Y_st=0;

 achat = find(mean(lat)<=10 & mean(lat)>=-10);    %latitude range
   lag = 2;                                       %lag size
 terms = 2;                                       %number of lagged terms
 np = terms + 1 + sum(terms:-1:1);                %total number of terms

  pDH_RHO  = zeros(np,nz,nl);                     %regression coefficients
        p3 = nans(nz,nval,nl);

     %Exclude mean fields. Added later
      p1_i = squeeze(p1_0(1,:,:))-p1_m;           %SLA
       p2l = p2_0-p2_m;                           %RHO
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   for nti = achat 
      nti
     for let = 1:nz               %loop in depth
   %Populate  
     P2 = p2l(1:let,:,nti);       %predictand  - fits the original field
     %P2 = p3(1:let,:,nti);       %predictand  - fits the prediction field
     X2 = zeros(let+1,nval); 
     X2(2:end,:) = P2;            %predictor - lagged fields selected below
     X2(1,:) = p1_i(:,nti)';      %predictor - surface dynamic height 
     
   %Standardize predictor
     X2    = X2 - mean(X2,2)*ones(1,nval);  %demean
     X2_st = std(X2,[],2); X2_st(X2_st == 0) = 1; X2 = X2./(X2_st*ones(1,nval)); %x/std
     
  %Standardize predictand. Do not demean
         Y = p2l(let,:,nti)';
      y_st = std(Y); y_st(y_st == 0) = 1; Y = Y./(y_st);    %y/std
  %Select lagged terms       
      p1_t = X2([ (let+1-lag) : -lag : max(2,let+1-lag*terms) 1],:)';  %select lagged fields
  %Calculate matrix of terms + intercept + interaction terms
         X = x2fx(p1_t(:,1:end),'interaction');  %interaction terms from regstat
        st = size(X,2); 
 %Multivariate regression      
         [beta,Sigma,E,CovB,logL] = mvregress(X ,Y);
         yfit = X*beta;
         Y_st(let,nti)    = y_st;                  %Store standard dev.
         p3(let,nani,nti) = yfit.*y_st;            %Store fitted
         pDH_RHO(1:st,let,nti) = beta;             %Store parameters
     end;
   rho_new = p3+p2_m;    %reconstructed field. Mean field added
   end
   
%   figure,contourf(squeeze(rho_new(:,25,:)),'edgecolor','none'),title('fit')
%   figure,contourf(squeeze(p2_0(:,25,:)),'edgecolor','none'),title('original')

 end

%% %%%%%%%%%%%%%%%%%%%%%
function [rho_new,C_RHO_DH] = reconstruct_fields_mvreg(lat,p1_0,p1_m,p2_0,p2_m,pDH,Y_st,modo)
%Reconstruct fields from coefficients calculated in the regression

[nz,nval,nl]=size(p2_0);
nani = 1:nval;
disp(['Reconstruct regression coefficients - ',upper(modo)]);
 
 achat = find(mean(lat)<=10 & mean(lat)>=-10);
   lag = 2;                       %lag size
 terms = 2;                       %number of terms
 
 C_RHO_DH  = nans(nz,nl);         %Correlation
 V_dep_lat = nans(nz,nl);         %pvalue

 
 pnan = isnan(squeeze(pDH(:,1,achat(1)))); pDH(pnan,:,:) = [];
        p3 = zeros(nz,nval,nl);
      p1_i = squeeze(p1_0(1,:,:))-p1_m;     %SLA
       p2l = p2_0-p2_m;                     %just for correlation
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 for nti = achat                     %loop in latitude
      nti
     for let = 1:nz                  %loop in depth
         P2 = p3(:,:,nti); 
     X2 = zeros(nz+1,nval); 
     X2(2:end,:) = P2;
     X2(1,:) = p1_i(:,nti)';
     X2 = X2 - mean(X2,2)*ones(1,nval);
     X2_st = std(X2,[],2); X2_st(X2_st == 0) = 1; X2 = X2./(X2_st*ones(1,nval));

         p1_t = X2([ max(let+1-lag) : -lag : max(2,let+1-lag*terms) 1],:)';
         X = x2fx(p1_t(:,1:end),'interaction');
         st = size(X,2);
 
         beta = pDH(:,let,nti);                 %parameters from regression
         Y = X*beta(1:st).*Y_st(let,nti)';      %Reconstruction (* std)
         p3(let,nani,nti) = Y;                  %Store reconstruction
         [C_RHO_DH(let,nti),V_dep_lat(let,nti)] = corr(squeeze(p2l(let,nani,nti))',Y);
        
     end
 end
 rho_new = p3 + p2_m;  %Add mean field
%figure,contourf(squeeze(rho_new(:,25,:)),'edgecolor','none')
%figure,contourf(squeeze(p2_0(:,25,:)),'edgecolor','none')
 return
end


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_regression_fit(sigma_new,rho,Pn,modo)
 h=figure;
 subplot(2,1,1)
 dep = 350; %700m deep
 pa = sigma_new(dep,:,1:end);pb = rho(dep,:,1:end);
                 plot(pb(:),pa(:),'.'),ylabel('XBT'),xlabel('Fitted');
                  title([modo,' at z=',num2str(Pn(dep)),' m'])
 subplot(2,1,2)
 dep = 41; %80 m deep;
 pa = sigma_new(dep,:,1:end);pb = rho(dep,:,1:end);
                 plot(pb(:),pa(:),'.'),ylabel('XBT'),xlabel('Fitted');
                  title([modo,' at z=',num2str(Pn(dep)),' m'])
                 pause(1)
save2eps(['compare_fit_regression_',modo,'_AR1.eps'],h,300)
end
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_fields(dep_ax8,lat_ax8,rho,rho_new,rho_rec)
%%% Plot both fieds for validation
disp('plotting fields ...')

foo=figure(5);
times = 25;
levs = 22.5:.5:27;
Pn = dep_ax8;
subplot(2,2,1)
  contourf(lat_ax8(times,:),-Pn,squeeze(rho(:,times,:)),levs),colorbar
  xlim([-10 15]),caxis([levs(1) levs(end)])
  title('\sigma_{\theta} (kg m^3 s^{-1}) - XBT')
  set(gca,'yticklabel',num2str(abs(get(gca,'ytick'))','%i'))
subplot(2,2,2)
  contourf(lat_ax8(times,:),-Pn,squeeze(rho_rec(:,times,:)),levs),colorbar
  xlim([-10 15]),caxis([levs(1) levs(end)])
  title('\sigma_{\theta} (kg m^3 s^{-1}) - Reconstructed')
  set(gca,'yticklabel',num2str(abs(get(gca,'ytick'))','%i'))
subplot(2,2,3)
  contourf(lat_ax8(times,:),-Pn,squeeze(rho(:,times,:)),levs),colorbar
  xlim([-10 15]),caxis([levs(1) levs(end)])
  title('\sigma_{\theta} (kg m^3 s^{-1}) - XBT')
  set(gca,'yticklabel',num2str(abs(get(gca,'ytick'))','%i'))
subplot(2,2,4)
  contourf(lat_ax8(times,:),-Pn,squeeze(rho_new(:,times,:)),levs),colorbar
  xlim([-10 15]),caxis([levs(1) levs(end)])
  title('\sigma_{\theta} (kg m^3 s^{-1}) - Fitted')
  set(gca,'yticklabel',num2str(abs(get(gca,'ytick'))','%i'))
save2eps('DH_RHO_valid_AR1.eps',foo,300)

end
%%
function plot_C_RHO_DH(C_RHO_DH,lat_ax8,Pn)
lat = lat_ax8(1,:);

h = figure(5);
clf;

contourf(lat,-Pn,C_RHO_DH,-1:.2:1),
caxis([-1 1]),
set(gca,'fontsize',14,'xlim',[-10 15]),
grid('on')
title('R(DH_0,\sigma_{\theta})')

save2eps('Correlation_depth_DH_SH_rho_AR1.eps',h,300)
end
